#include <iostream>
#include <string>
using namespace std;

#include <SDL.h>
#include <SDL_image.h>

#include "headers.h"
#include <GL/glu.h>
#include "textures.h"
#include "mathlib.h"
#include "objets.h"
#include "Vecteur3d.h"
#include "Metaball.h"
#include "champ.h"
#include "object.hpp"

#define CODE_ERREUR 1
#define CODE_OK     0

# define PI 3.141592654

// Affichage-------------
#define LARGEUR 800
#define HAUTEUR 600
#define ECRAN_BPP 24
//#define FULLSCREEN
#define FOV 45.0f

// SDL--------------------
long temps;
SDL_Surface *surface=NULL;

// material
GLfloat ambientMaterial[]={0.11f,0.06f,0.11f,1.0f};
GLfloat diffuseMaterial[]={0.43f,0.47f,0.54f,1.0f};
GLfloat specularMaterial[]={0.33f,0.33f,0.52f,1.0f};
GLfloat emissionMaterial[]={0.0f,0.0f,0.0f,0.0f};
GLfloat shininessMaterial=10.0f;

// 2D
GLuint logotexture;
SDL_Surface * sdllogotexture;

// IHM
bool go=true;

/********************************************************************************/
// INIT
/********************************************************************************/
SDL_Surface* initSDL()
{
  SDL_Surface* surface=NULL;

  int res=SDL_Init(SDL_INIT_VIDEO);

  if (res==-1)
    {
      cout << "Res = -1\n";
      SDL_Quit();
    }

  const SDL_VideoInfo* videoInfo=NULL;
  videoInfo=SDL_GetVideoInfo();

  if (!videoInfo)
    {
      cout<<"Pas de videoInfo!\n";
      SDL_Quit();
    }

  int videoParams=SDL_OPENGL|SDL_GL_DOUBLEBUFFER|SDL_HWPALETTE;

#ifdef FULLSCREEN
  videoParams|=SDL_FULLSCREEN;
#endif

  videoParams|=((videoInfo->hw_available)?SDL_HWSURFACE:SDL_SWSURFACE);
  if (videoInfo->blit_hw)
    videoParams|=SDL_HWACCEL;

  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);

  surface=SDL_SetVideoMode(LARGEUR,HAUTEUR,ECRAN_BPP,videoParams);

  if (!surface)
    {
      cout<<"Pas de surface !\n";
      SDL_Quit();
    }

  return surface;
}

/********************************************************************************/
// OPENGL
/********************************************************************************/

void redimensionner(int largeur, int hauteur)
{
  largeur=max(largeur,1);
  hauteur=max(hauteur,1);

  GLfloat rapport=(GLfloat)largeur/(GLfloat)hauteur;

  glViewport(0,0,(GLint)largeur,(GLint)hauteur);  
  glMatrixMode(GL_PROJECTION);
  gluPerspective(FOV,rapport,0.1f,1000.0f);  
  glMatrixMode(GL_MODELVIEW);
}

void ViewOrtho(void)
{
  int largeur=max(LARGEUR,1);
  int hauteur=max(HAUTEUR,1);

  GLfloat rapport=(GLfloat)largeur/(GLfloat)hauteur;

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho( 0, LARGEUR , HAUTEUR , 0, -1, 1 );
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void ViewPerspective()
{
  int largeur=max(LARGEUR,1);
  int hauteur=max(HAUTEUR,1);

  GLfloat rapport=(GLfloat)largeur/(GLfloat)hauteur;

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective(FOV,rapport,0.1f,1000.0f);  
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
}

/********************************************************************************/
// AFFICHAGE
/********************************************************************************/

void drawGL_Repere(void)
{
  glPushMatrix();

  glScalef(1.0f,1.0f,1.0f);
  
  glBegin(GL_LINES);

  glColor4f(1.0f,0.0f,0.0f,1.0f);
  glVertex3f(0.0f,0.0f,0.0f);
  glVertex3f(1.0f,0.0f,0.0f);

  glColor4f(0.0f,1.0f,0.0f,1.0f);
  glVertex3f(0.0f,0.0f,0.0f);
  glVertex3f(0.0f,1.0f,0.0f);

  glColor4f(0.0f,0.0f,1.0f,1.0f);
  glVertex3f(0.0f,0.0f,0.0f);
  glVertex3f(0.0f,0.0f,1.0f);

  glEnd();

  glPopMatrix();
}

/********************************************************************************/
// METABALLS
/********************************************************************************/

void init2D(void)
{
  cout<<"Init2D...\n";

  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);

  // Logo
  glGenTextures (1, &logotexture);
  glBindTexture (GL_TEXTURE_2D, logotexture);
  
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  
  sdllogotexture=IMG_Load("./media/logo.png");
  glTexImage2D (GL_TEXTURE_2D, 0, GL_RGBA, sdllogotexture->w, sdllogotexture->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, sdllogotexture->pixels);  
  
  glDisable(GL_BLEND);
}

void affichage2D(void)
{
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  
  glDisable(GL_DEPTH_TEST);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE);
  glEnable(GL_BLEND);

  // Logo
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,logotexture);
  
  glBegin(GL_QUADS);  

  glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
  
  glTexCoord2f(0,0);
  glVertex2f(LARGEUR-sdllogotexture->w,255);
  
  glTexCoord2f(0,1);
  glVertex2f(LARGEUR-sdllogotexture->w,255+sdllogotexture->h);
  
  glTexCoord2f(1,1);
  glVertex2f(LARGEUR,255+sdllogotexture->h);
  
  glTexCoord2f(1,0);
  glVertex2f(LARGEUR,255);

  glEnd();
  
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_DEPTH_TEST);
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
}

/********************************************************************************/
// MAIN
/********************************************************************************/

int main()
{
  int fini=1;
  int frames=0;
  long tempsDebut=SDL_GetTicks();
  float fps=0.0f;
  Object *un_objet;

  SDL_Event evenement;
  surface=initSDL();
  init2D();

  un_objet=new Object();
  un_objet->test();
  
  redimensionner(LARGEUR,HAUTEUR);
  
  while(fini!=0)
    {
      temps=SDL_GetTicks();
      frames++;
      
      while(SDL_PollEvent(&evenement))
	{
	  switch(evenement.type)
	    {
	    case SDL_KEYDOWN:
	      go=!go;
	      if (evenement.key.keysym.sym == SDLK_ESCAPE)
		fini=0;
	      break;
	    case SDL_QUIT:
	      fini=0;
	      break;
	    default:
	      break;
	    }
	}

      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

      // Affichage de la 3D
      ViewPerspective();
      glEnable(GL_DEPTH_TEST);
      glTranslatef(0.0f,0.0f,-2.0f);
      
      drawGL_Repere();

      // Affichage des 2D
      ViewOrtho();
      affichage2D();

      if (SDL_GetTicks()-tempsDebut>1000)
	{
	  fps=1000.0f * (float)frames / (float) (SDL_GetTicks()-tempsDebut);
	  frames=0;
	  tempsDebut=SDL_GetTicks();
	  
	  cout<<fps<<" fps\n";

	}
      
      SDL_GL_SwapBuffers();
    }

  SDL_Quit();
  return CODE_OK;
}

